////////////////////////////////////////////////////////////////////// 
// extras/pedstats_pdf.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday March 19, 2003
// 
 
// This file was written by Jan Wigginton
//
 
#include "pedstats.h"
#include "pedstats_pdf.h"

// externally defined functions and variables
//

extern String pedName;
extern Pedigree ped;
extern int affection_filter, trait_filter, covariate_filter;

void GraphFamilyStatistics(PDF & pdf)
   {
   Vector familyMembers;
   Vector familyFounders;
   Vector familyGenerations;
   Vector familyNonFounders;

   int individuals = 0;

   int min_gens, max_gens ;
   int min_founders, max_founders ;
   int min_nonfounders, max_nonfounders;
   int min_members, max_members;

   for (int f = 0; f < ped.familyCount; f++)
      {
      if (f == 0)
         {
         min_founders = max_founders = ped.families[f]->founders;
         min_nonfounders = max_nonfounders = ped.families[f]->count - ped.families[f]->founders;
         min_members = max_members = ped.families[f]->count;
         }
      familyFounders.Push(ped.families[f]->founders);
      max_founders = max(max_founders, ped.families[f]->founders);
      min_founders = min(min_founders, ped.families[f]->founders);

      int non_founders = ped.families[f]->count  - ped.families[f]->founders;
      familyNonFounders.Push(non_founders);
      min_nonfounders = min(min_nonfounders, non_founders);
      max_nonfounders = max(max_nonfounders, non_founders);

      familyMembers.Push(ped.families[f]->count);
      max_members = max(max_members, ped.families[f]->count);
      min_members = min(min_members, ped.families[f]->count);

      int gens = 0;
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         gens = max(Generation(ped.persons[i]) + 1, gens);
      familyGenerations.Push(gens);

      if (f == 0)
         min_gens = max_gens = gens;
      else
        {
        max_gens = max(max_gens, gens);
        min_gens = min(min_gens, gens);
        }

      individuals += ped.families[f]->count;
      }

    pdf.page.OpenPage();

    // settings for family member charts
    PDFHistogram family_chart;
    family_chart.useLegend = false;
    family_chart.yAxis.useDiscreteValues = true;

    family_chart.yAxis.label = "Number of pedigrees";
    family_chart.Dimension(1, ped.familyCount);

    // overall
    family_chart.SetDataValues(0, familyMembers);
    family_chart.SetSeriesColor(0, 255, 0, 0);
    family_chart.SetCategoricalBars(true, min_members, max_members);

    family_chart.title = "Individuals";
    family_chart.xAxis.label = "Number of family members";

    family_chart.DrawInUpperLeft(pdf);

    // founders
    family_chart.SetDataValues(0, familyFounders);
    family_chart.SetCategoricalBars(true, min_founders, max_founders);
    family_chart.SetSeriesColor(0, 0, 255, 255);

    family_chart.title = "Founders";
    family_chart.xAxis.label = "Number of founders";

    family_chart.DrawInLowerRight(pdf);

    // non-founders
    family_chart.SetDataValues(0, familyNonFounders);
    family_chart.SetCategoricalBars(true, min_nonfounders, max_nonfounders);
    family_chart.SetSeriesColor(0, 0, 255, 0);

    family_chart.title = "Non-founders";
    family_chart.xAxis.label = "Number of non-founders";

    family_chart.DrawInLowerLeft(pdf);

    // number of generations
    family_chart.SetDataValues(0, familyGenerations);
    family_chart.SetCategoricalBars(true, min_gens, max_gens);
    family_chart.SetSeriesColor(0, 0, 0, 255);

    family_chart.title = "Generations";
    family_chart.xAxis.label = "Number of generations";

    family_chart.DrawInUpperRight(pdf);

    // make page header
    int ct = ped.familyCount;
    String sub_title;
    sub_title.printf("( %d individual%s in %d famil%s )",
                     individuals, individuals != 1 ? "s" : "",
                     ct, ct != 1 ? "ies" : "y");

    MakePageHeader(pdf, "Summary of Family Structure", sub_title);

    pdf.page.ClosePage();
   }

void GraphTraitStatistics(PDF & pdf)
   {
   if (Person::traitCount == 0) return;

   for (int trait = 0; trait < Person::traitCount; trait++)
      GraphTrait(pdf, trait_filter = trait);
   }

void GraphTrait(PDF & pdf, int t)
   {
   int    tcount = 0;

   Vector allTraitValues;
   Vector founderTraitValues, nonFounderTraitValues;
   Vector femaleTraitValues, maleTraitValues;

   double trait_value;

   for (int i = 0; i < ped.count; i++)
      {
      if ( !ped[i].isPhenotyped(t) ) continue;

      tcount++;
      trait_value = ped[i].traits[t];

      allTraitValues.Push(trait_value);

      if (ped[i].isFounder())
          founderTraitValues.Push(trait_value);
      else
          nonFounderTraitValues.Push(trait_value);

      if (ped[i].sex == SEX_MALE)
          maleTraitValues.Push(trait_value);
      else if (ped[i].sex == SEX_FEMALE)
          femaleTraitValues.Push(trait_value);
      }

    pdf.page.OpenPage();

    PDFHistogram trait_value_chart;
    trait_value_chart.xAxis.label = "Trait value";
    trait_value_chart.yAxis.label = "Individuals";
    trait_value_chart.stacked = false;

    GraphVarOverall(pdf, trait_value_chart, allTraitValues);
    GraphVarBySex(pdf, trait_value_chart, femaleTraitValues, maleTraitValues, true);
    GraphVarByFounder(pdf, trait_value_chart, founderTraitValues, nonFounderTraitValues, true);
    GraphVarBySibs(pdf, false);

    String title;
    title.printf("Distribution of Values for %.20s Trait", (const char *) ped.traitNames[t]);

    String sub_title;
    if (tcount > 0)
       sub_title.printf( "( Mean: %.2f Variance: %.2f )",
        allTraitValues.Average(), (tcount <= 1) ? 0.0 : allTraitValues.Var());

    MakePageHeader(pdf, title, sub_title);

    pdf.page.ClosePage();
   }

void GraphCovariateStatistics(PDF & pdf)
   {
   if (Person::covariateCount == 0) return;

   for (int covariate = 0; covariate < Person::covariateCount; covariate++)
     GraphCovariate(pdf, covariate_filter = covariate);
   }

void GraphCovariate(PDF & pdf, int c)
   {
   Vector allCovValues;
   Vector founderCovValues, nonFounderCovValues;
   Vector maleCovValues, femaleCovValues;

   int    tcount = 0;
   double cov_value;

   for (int i = 0; i < ped.count; i++)
      {
      if ( !ped[i].isControlled(c) ) continue;
      tcount++;

      cov_value = ped[i].covariates[c];

      allCovValues.Push(cov_value);

      if (ped[i].isFounder())
         founderCovValues.Push(cov_value);
      else
         nonFounderCovValues.Push(cov_value);

      if (ped[i].sex == SEX_MALE)
         maleCovValues.Push(cov_value);
      else if (ped[i].sex == SEX_FEMALE)
         femaleCovValues.Push(cov_value);
      }

   pdf.page.OpenPage();

   PDFHistogram covariate_chart;
   covariate_chart.xAxis.label = "Covariate value";
   covariate_chart.yAxis.label = "Individuals";
   covariate_chart.stacked = false;

   GraphVarOverall(pdf, covariate_chart, allCovValues);
   GraphVarBySex(pdf, covariate_chart, femaleCovValues, maleCovValues, true);
   GraphVarByFounder(pdf, covariate_chart, founderCovValues, nonFounderCovValues, true);
   GraphVarBySibs(pdf, true);

   // page header
   String sub_title = "", title;

   title.printf("Distribution of Values for: %-15.15s", (const char *) ped.covariateNames[c]);

   if (tcount > 0)
     sub_title.printf("( Mean: %.2f Variance: %.2f )", allCovValues.Average(),
                     tcount <= 1 ? 0.0 : allCovValues.Var());

   MakePageHeader(pdf, title, sub_title);

   pdf.page.ClosePage();
   }

void GraphVarOverall(PDF & pdf, PDFHistogram & chart, Vector & values)
   {
   chart.Dimension(1, values.dim);
   chart.SetDataValues(0, values);

   chart.SetSeriesColor(0, 0, 0, 255);
   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.0);
   chart.narrowBars = true;
   chart.useLegend = false;

   chart.title = "Overall distribution";

   chart.DrawInUpperLeft(pdf);
   chart.narrowBars = false;
   }

void GraphVarBySex(PDF & pdf, PDFHistogram & chart, Vector & v_female, Vector & v_male,
                   bool do_subtitle)
   {
   int max_dim = max(v_female.dim, v_male.dim);
   chart.Dimension(2, max_dim);
   chart.SetDataValues(0, v_female);
   chart.SetDataValues(1, v_male);

   chart.SetSeriesColor(0, 0, 0, 255);
   chart.SetSeriesColor(1, 0, 255, 0);
   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.1);
   chart.useLegend = true;

   chart.title = "Distribution by sex ";
   if (do_subtitle)
      {
      int male_ct = v_male.SafeCount();
      int female_ct = v_female.SafeCount();

      chart.subTitle.printf("( %d %s, %d %s )", male_ct, (male_ct != 1) ?  "males" : "male",
                                female_ct, (female_ct != 1) ? "females": "female" );
      }

   chart.SetSeriesLabel(0, "Females");
   chart.SetSeriesLabel(1, "Males");

   chart.DrawInUpperRight(pdf);
   chart.subTitle = "";
   }

void GraphVarByFounder(PDF & pdf, PDFHistogram & chart, Vector & v_founder, Vector & v_nonfounder,
                        bool do_subtitle)
   {
   int max_dim = max(v_founder.dim, v_nonfounder.dim);
   chart.Dimension(2, max_dim);

   chart.SetDataValues(0, v_founder);
   chart.SetDataValues(1, v_nonfounder);

   if (!chart.xAxis.useDiscreteValues) chart.SetBarSpacing(0.1);

   chart.title = "Distribution by founder status";

   if (do_subtitle)
      {
      int founder_ct = v_founder.SafeCount();
      int nonfounder_ct = v_nonfounder.SafeCount();


      chart.subTitle.printf("( %d %s, %d %s )",
                      founder_ct,
                      founder_ct != 1 ?  "founders" : "founder",
                      nonfounder_ct,
                      nonfounder_ct != 1 ? "non-founders": "non-founder");
      }

   chart.SetSeriesLabel(0, "Founder");
   chart.SetSeriesLabel(1, "Non-Founder");

   chart.SetSeriesColor(0, 0, 0 , 255);
   chart.SetSeriesColor(1, 0, 255, 0);

   chart.DrawInLowerRight(pdf);
   chart.subTitle = "";
   }

void GraphVarBySibs(PDF & pdf, bool is_covariate)
   {
   PDFLineChart chart;

   chart.title = "Sib-Sib plot";
   chart.xAxis.label = is_covariate ? "Sib covariate value" :"Sib trait value";
   chart.yAxis.label = is_covariate ? "Cosib covariate value" : "Cosib trait value";

   int sib_pairs = 0;
   double r_sib = is_covariate ?
      GetSibData(FilterOnCovariate, chart, sib_pairs, is_covariate) :
      GetSibData(FilterOnTrait, chart, sib_pairs, is_covariate);

   String temp;
   if (r_sib != _NAN_)
      {
      temp.printf("( %d phenotyped sib %s, r = %5.2lf )", sib_pairs / 2,
                   sib_pairs > 2 ? "pairs" : "pair",
                   r_sib);
      chart.subTitle += temp;
      }
   else
      {
      temp.printf("( No phenotyped sib pairs )");
      chart.subTitle += temp;
      }

   chart.ShowLine(0, false);
   chart.ShowMarker(0,true);

   if (sib_pairs <= 4)
     chart.SetMarkerRadius(0, 5.0);
   else if (sib_pairs < 20)
      chart.SetMarkerRadius(0, 4.0);
   else if (sib_pairs < 40)
     chart.SetMarkerRadius(0, 2.0);

   chart.useLegend = false;

   chart.DrawInLowerLeft(pdf);
   }

double GetSibData(bool (*filter)(Person &, Person &), PDFLineChart & chart,
   int & sib_pairs, bool is_covariate)
   {
   Vector x_vals;
   Matrix y_vals;

   y_vals.Dimension(1, 0);
   x_vals.Dimension(0);

   for (int f = 0; f < ped.familyCount; f++)
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         for (int j = ped.families[f]->first; j <= ped.families[f]->last; j++)
            {
            if (j == i) continue;
            if (filter == NULL || filter(ped[i], ped[j]))
               if (ped[i].isSib(ped[j]))
                 {
                 sib_pairs++;

                 if (is_covariate)
                    {
                    x_vals.Push(ped[i].covariates[covariate_filter]);
                    y_vals[0].Push(ped[j].covariates[covariate_filter]);
                    }
                 else
                    {
                    x_vals.Push(ped[i].traits[trait_filter]);
                    y_vals[0].Push(ped[j].traits[trait_filter]);
                    }
                 }
            }

   x_vals.Dimension(sib_pairs);
   y_vals.Dimension(1, sib_pairs);
   chart.Dimension(1, sib_pairs);
   chart.SetDataValues(x_vals, y_vals);

   double r_sib = _NAN_;

   if (sib_pairs > 1)
      {
      double s_xx = x_vals.SumSquares() - sib_pairs * x_vals.Average() * x_vals.Average();
      double s_yy = y_vals[0].SumSquares() - sib_pairs * y_vals[0].Average() * y_vals[0].Average();
      double s_xy = x_vals.InnerProduct(y_vals[0]) - sib_pairs * y_vals[0].Average() * x_vals.Average();
      double denom = s_xx * s_yy;
      if (denom > 0.0) r_sib =  s_xy / sqrt(denom);
      }

   return r_sib;
   }

void GraphMarkerStatistics(PDF & pdf)
   {
   if (Person::markerCount == 0) return;

   for (int m = 0; m < Person::markerCount; m++)
      GraphMarker(pdf, m);
   }

void GraphMarker(PDF & pdf, int m)
   {
   int mGenotypes = 0, mPossible = 0;
   int lastAllele = 0, firstAllele = 100000;
   int ct_males = 0, ct_females = 0;
   int ct_founders = 0, ct_nonfounders = 0;

   Vector overallAlleles;
   Vector founderAlleles, nonFounderAlleles;
   Vector maleAlleles, femaleAlleles;
   Vector hetCounts, homCounts;

   for (int i = 0; i < ped.count; i++)
      {
      mPossible  ++;
      if ( !ped[i].isGenotyped(m) ) continue;
      mGenotypes++;

      int m0 = ped[i].markers[m][0];
      int m1 = ped[i].markers[m][1];

      overallAlleles.Push(m0);
      overallAlleles.Push(m1);

      if (ped[i].isFounder())
         {
         ct_founders++;
         founderAlleles.Push(m0);
         founderAlleles.Push(m1);
         }
      else
         {
         ct_nonfounders++;
         nonFounderAlleles.Push(m0);
         nonFounderAlleles.Push(m1);
         }

      if (ped[i].sex == SEX_MALE)
         {
         ct_males++;
         maleAlleles.Push(m0);
         maleAlleles.Push(m1);
         }
      else if (ped[i].sex == SEX_FEMALE)
         {
         ct_females++;
         femaleAlleles.Push(m0);
         femaleAlleles.Push(m1);
         }

      firstAllele = min(min(m0, m1), firstAllele);
      lastAllele  = max(max(m0, m1), lastAllele);

      if (ped.chromosomeX && ped[i].sex == SEX_MALE) continue;

      if ( ped[i].markers[m].isHeterozygous())
         hetCounts.Push(2.0);
      else if (ped[i].markers[m].isHomozygous())
         homCounts.Push(1.0);
      }

   pdf.page.OpenPage();

   PDFHistogram marker_chart;

   // Overall distribution
   marker_chart.stacked = false;
   marker_chart.SetUseSeriesPercentages(true);
   marker_chart.SetCategoricalBars(true, firstAllele, lastAllele);
   marker_chart.yAxis.maxMax = 1.1;

   marker_chart.xAxis.label = "Allele number";
   marker_chart.yAxis.label = "Allele frequency";

   GraphVarOverall(pdf, marker_chart, overallAlleles);

   // By sex distribution
   marker_chart.subTitle.printf("( %d %s, %d %s )",
                                 ct_males, (ct_males != 1) ? "males" : "male",
                                 ct_females, (ct_females != 1) ? "females" : "female");
   GraphVarBySex(pdf, marker_chart, femaleAlleles, maleAlleles, false);

   // By founder status distribution
   marker_chart.subTitle.printf("( %d %s, %d %s )", ct_founders,
                                   ct_founders != 1 ? "founders" : "founder",
                                   ct_nonfounders,
                                   ct_nonfounders != 1 ? "non-founders" : "non-founder");
   GraphVarByFounder(pdf, marker_chart, founderAlleles, nonFounderAlleles, false);

   // Informativeness chart
   marker_chart.Dimension(2, max(hetCounts.dim, homCounts.dim));
   marker_chart.SetDataValues(0, homCounts);
   marker_chart.SetDataValues(1, hetCounts);

   marker_chart.SetCategoricalBars(true, 1.0, 2.0);
   marker_chart.SetUseTotalPercentages(true);
   marker_chart.stacked = true;

   marker_chart.xAxis.useTickLabels = false;
   marker_chart.xAxis.useTicks = false;
   marker_chart.xAxis.SetMin(0.5);
   marker_chart.xAxis.SetMax(2.5);

   marker_chart.SetSeriesColor(0, 255, 0, 0);
   marker_chart.SetSeriesColor(1, 0, 255, 0);

   marker_chart.SetSeriesLabel(0, "Homozygote");
   marker_chart.SetSeriesLabel(1, "Heterozygote");

   marker_chart.title = "Informativeness";
   if (ped.chromosomeX) marker_chart.subTitle = "( X-linked marker: Among females )";
   marker_chart.yAxis.label = "Percentage of population";
   marker_chart.xAxis.label = "";

   marker_chart.DrawInLowerLeft(pdf);
   marker_chart.subTitle = "";

   String title, sub_title;
   title.printf("Marker Allele Frequencies for: %-15.15s",   (const char *) Person::markerNames[m]);

   sub_title.printf("( %d of %d %s genotyped )", mGenotypes, mPossible,
                      mPossible != 1 ? "individuals" : "individual");

   MakePageHeader(pdf, title, sub_title);

   pdf.page.ClosePage();
   }

void GraphAffectionStatistics(PDF & pdf)
   {
   for (affection_filter = 0; affection_filter < ped.affectionCount; affection_filter++)
      GraphAffection(pdf, affection_filter);
   }

void GraphAffection(PDF & pdf, int index)
   {
   if (Person::affectionCount == 0) return;

   Vector nonFounderAff, founderAff;
   Vector maleAff, femaleAff;
   Vector overallAff;

   int    tcount = 0, tpossible = 0;

   for (int i = 0; i < ped.count; i++)
      {
      tpossible  ++;
      if ( !ped[i].isDiagnosed(index) ) continue;
      tcount ++;

      double aff_stat = ped[i].affections[index];

      overallAff.Push(aff_stat);

      if (ped[i].isFounder())
         founderAff.Push(aff_stat);
      else
         nonFounderAff.Push(aff_stat);

      if (ped[i].sex == SEX_MALE)
         maleAff.Push(aff_stat);
      else if (ped[i].sex == SEX_FEMALE)
         femaleAff.Push(aff_stat);
      }

   pdf.page.OpenPage();

   PDFHistogram affection_chart;

   affection_chart.stacked = false;
   affection_chart.SetCategoricalBars(true, 1.0, 2.0);
   affection_chart.yAxis.useDiscreteValues = true;

   affection_chart.yAxis.label = "Individuals";
   affection_chart.xAxis.label = "Affection status";

   GraphVarOverall(pdf, affection_chart, overallAff);
   GraphVarBySex(pdf, affection_chart, femaleAff, maleAff, true);
   GraphVarByFounder(pdf, affection_chart, founderAff, nonFounderAff, true);
   GraphSibAffection(pdf, affection_chart, index);

   String sub_title, title;
   title.printf("Affection Status Statistics for: %-15.15s",
                  (const char *) Person::affectionNames[index]);

   sub_title.printf("( %d of %d %s diagnosed )",tcount, tpossible,
                    tpossible != 1 ? "individuals" : "individual");

   MakePageHeader(pdf, (const char *) title, (const char *) sub_title);

   pdf.page.ClosePage();
   }

void GraphSibAffection(PDF & pdf, PDFHistogram & chart, int index)
   {
   Vector discordantSibs;
   Vector concordantAffSibs;
   Vector concordantUnaffSibs;

   int sib_pairs = GetSibAffectionData(discordantSibs, concordantAffSibs, concordantUnaffSibs, index);

   int max_dim = max(discordantSibs.dim, max(concordantAffSibs.dim, concordantUnaffSibs.dim));
   chart.Dimension(3, max_dim);

   chart.SetDataValues(0, concordantUnaffSibs);
   chart.SetDataValues(1, discordantSibs);
   chart.SetDataValues(2, concordantAffSibs);

   chart.stacked = true;
   chart.SetCategoricalBars(true, 1.0, 3.0);
   chart.xAxis.useTicks = false;
   chart.xAxis.useTickLabels = false;

   chart.title = "Concordant/Discordant sib pairs";
   if (sib_pairs > 0)
      chart.subTitle.printf("( %d diagnosed sib %s )", sib_pairs,
                              sib_pairs != 1 ? "pairs" : "pair");
   else
     chart.subTitle.printf("( No diagnosed sib pairs )");

   chart.yAxis.label = "Sib Pairs";

   chart.SetSeriesLabel(0, "Unaffected");
   chart.SetSeriesLabel(1, "Discordant");
   chart.SetSeriesLabel(2, "Affected");

   chart.SetSeriesColor(0, 0, 255, 0);
   chart.SetSeriesColor(1, 255, 255, 0);
   chart.SetSeriesColor(2, 255, 0, 0);

   chart.DrawInLowerLeft(pdf);
   }

int GetSibAffectionData(Vector & d_sibs, Vector & ca_sibs, Vector & cu_sibs, int index)
   {
   int sib_pairs = 0;

   for (int f = 0; f < ped.familyCount; f++)
      for (int i = ped.families[f]->first; i <= ped.families[f]->last; i++)
         for (int j = i + 1; j <= ped.families[f]->last; j++)
            {
            if (j == i) continue;
            if (ped[i].isSib(ped[j]))
               {
               int stat_1 = ped[i].affections[index];
               int stat_2 = ped[j].affections[index];

               if ( stat_1 == 1 && stat_2 == 1)
                  {
                  cu_sibs.Push(1.0);
                  sib_pairs++;
                  }
               else if ( stat_1 == 2 && stat_2 == 2)
                  {
                  ca_sibs.Push(3.0);
                  sib_pairs++;
                  }
               else if ( stat_1 == 2 && stat_2 == 1 || stat_1 == 1 && stat_2 == 2)
                  {
                  d_sibs.Push(2.0);
                  sib_pairs++;
                  }
               }
            }
   return sib_pairs;
   }

void MakePageHeader(PDF & pdf, const char *title, const char *sub_title)
   {
   pdf.page.hTextAlignment = taCenter;
   pdf.page.vTextAlignment = taMiddle;

   pdf.page.SetFontSize(16);
   pdf.page.WriteText(pdf.page.GetWidth() * 0.5, pdf.page.GetHeight() * 0.95, title);

   pdf.page.SetFontSize(10);
   pdf.page.WriteText(pdf.page.GetWidth() * 0.5, pdf.page.GetHeight() * 0.91, sub_title);
   }





 
